from unittest import TestCase
from opensearch import Description

class DescriptionTests(TestCase):

    # just grab the description once for all these tests
    url = 'file:test/description.xml'
    desc = Description(url)

    def test_url(self):
        self.assertEqual(self.desc.url,
            'http://www.koders.com/?s={searchTerms}&p={startPage}&output=rss')

    def test_format(self):
        self.assertEqual(self.desc.format, 
            'http://a9.com/-/spec/opensearchrss/1.0/')

    def test_shortname(self):
        self.assertEqual(self.desc.shortname, 'Koders')

    def test_longname(self):
        self.assertEqual(self.desc.longname, 'Koders Source Code Search')

    def test_description(self):
        self.assertEqual(self.desc.description, 'Search open source code on Koders.com. Supports languages such as java, c, c++, c#, php, perl, python, and many others.')

    def test_tags(self):
        self.assertEqual(self.desc.tags, ['koders', 'source', 'code', 
            'open-source', 'programming', 'software'])

    def test_image(self):
        self.assertEqual(self.desc.image,
            'http://www.koders.com/images/koders_icon64x64.gif')

    def test_samplesearch(self):
        self.assertEqual(self.desc.samplesearch, 'calculator')

    def test_developer(self):
        self.assertEqual(self.desc.developer, 'Koders Inc.')

    def test_contact(self):
        self.assertEqual(self.desc.contact, 'webmaster@koders.com')

    def test_attribution(self):
        self.assertEqual(self.desc.attribution,
            '&copy; 2005, Koders Inc., All Rights Reserved.')

    def test_syndicationright(self):
        self.assertEqual(self.desc.syndicationright, 'open')

    def test_adultcontent(self):
        self.assertEqual(self.desc.adultcontent, False)



import re
import urllib
import urlparse

class Query:
    """Represents an opensearch query. Used internally by the Client to 
    construct an opensearch url to request. Really this class is just a 
    helper for substituting values into the macros in a format. 

    format = 'http://beta.indeed.com/opensearch?q={searchTerms}&start={startIndex}&limit={count}'
    q = Query(format)
    q.searchTerms('zx81')
    q.startIndex = 1
    q.count = 25
    print q.to_url()
    """

    standard_macros = ['searchTerms','count','startIndex','startPage', 
        'language', 'outputEncoding', 'inputEncoding']

    def __init__(self, format):
        """Create a query object by passing it the url format obtained
        from the opensearch Description.
        """
        self.format = format
        self.url_parts = urlparse.urlparse(format)
        qsl = urlparse.parse_qsl(self.url_parts[4])
        self.params = []

        for key, value in qsl:
            self.params.append(Param(key, value))

    def url(self):
        # build new query string list substituting any macros
        # TODO: make sure required params are filled in
        qsl = []
        for param in self.params:
            if param.macro and hasattr(self, param.macro):
                qsl.append((param.name, getattr(self, param.macro)))
            elif not param.macro:
                qsl.append((param.name, param.value))

        # create new url using the query string list
        url_parts = list(self.url_parts)
        url_parts[4] = urllib.urlencode(qsl)
        return urlparse.urlunparse(tuple(url_parts))

    def has_macro(self, macro_name):
        for param in self.params:
            if param.macro == macro_name:
                return True
        return False

class Param:

    def __init__(self, name, value):
        self.name = name
        self.value = value 

        # this ugly regex is looking to capture 'macro' in '{macro}',  
        # '{prefix:macro}' and '{macro?}' ... sorry
        match = re.match('^{((.+):)?(.+?)(\?)?}$', value)

        if match:
            self.prefix = match.group(2)
            if self.prefix:
                self.macro = self.prefix + "__" + match.group(3)
            else:
                self.macro = match.group(3)
            self.optional = match.group(4) == "?"
        else:
            self.prefix = None
            self.macro = None
            self.optional = False

    def __repr__(self):
        return "param(name=%s, prefix=%s, macro=%s, optional=%s value=%s" % (self.name, self.prefix, self.macro, self.optional, self.value)




from unittest import TestCase
from opensearch import Query, Param

class QueryTests(TestCase):

    template = "http://www.koders.com/?s={searchTerms}&p={startPage}&output=rss&c={count}&i={startIndex}"

    def test_searchTerms(self):
        q = Query(QueryTests.template)
        q.searchTerms = 'zx81'
        self.assertEqual(q.url(), 'http://www.koders.com/?s=zx81&output=rss')

    def test_count(self):
        q = Query(QueryTests.template)
        q.count = 25
        self.assertEqual(q.url(), 'http://www.koders.com/?output=rss&c=25')

    def test_startPage(self):
        q = Query(QueryTests.template)
        q.startPage = 5
        self.assertEqual(q.url(), 'http://www.koders.com/?p=5&output=rss')

    def test_startIndex(self):
        q = Query(QueryTests.template)
        q.startIndex = 5
        self.assertEqual(q.url(), 'http://www.koders.com/?output=rss&i=5')

    def test_multiple(self):
        q = Query(QueryTests.template)
        q.searchTerms = 'zx81'
        q.startIndex = 15
        q.count = 15
        self.assertEqual(q.url(), 
            'http://www.koders.com/?s=zx81&output=rss&c=15&i=15')

    def test_extra_param(self):
        template = "http://www.koders.com/?s={searchTerms}&p={startPage}&output=rss&c={count}&i={startIndex}&key={key}"
        q = Query(template)
        self.assertTrue(q.has_macro('key'))
        q.searchTerms = 'zx81'
        q.key = 'abc123'
        self.assertEqual(q.url(), 'http://www.koders.com/?s=zx81&output=rss&key=abc123')

    def test_prefix(self):
        template = "http://worldcat.org/webservices/catalog/search/worldcat/opensearch?q={searchTerms}&start={startIndex?}&count={resultSize?}&format=atom&wskey={wcapi:wskey}&cformat={wcapi:cformat?}"
        q = Query(template)
        self.assertTrue(q.has_macro('searchTerms'))
        self.assertTrue(q.has_macro('wcapi__wskey'))
        q.searchTerms = "computer"
        q.wcapi__wskey = "abc123"
        self.assertEqual(q.url(), 'http://worldcat.org/webservices/catalog/search/worldcat/opensearch?q=computer&format=atom&wskey=abc123')

    def test_param(self):
        p = Param("foo", "bar")
        self.assertEqual(p.name, "foo")
        self.assertEqual(p.value, "bar")
        self.assertEqual(p.macro, None)
        self.assertEqual(p.optional, False)

    def test_macro(self):
        p = Param("foo", "{bar}")
        self.assertEqual(p.name, "foo")
        self.assertEqual(p.value, "{bar}")
        self.assertEqual(p.macro, "bar")
        self.assertEqual(p.optional, False)

    def test_prefix_macro(self):
        p = Param("foo", "{bar:baz}")
        self.assertEqual(p.name, "foo")
        self.assertEqual(p.value, "{bar:baz}")
        self.assertEqual(p.macro, "bar__baz")
        self.assertEqual(p.optional, False)




from urllib2 import urlopen, Request
from xml.dom.minidom import parse
from url import URL

class Description:
    """A class for representing OpenSearch Description files.
    """

    def __init__(self, url="", agent=""):
        """The constructor which may pass an optional url to load from.

        d = Description("http://www.example.com/description")
        """
        self.agent = agent
        if url: 
            self.load(url)


    def load(self, url):
        """For loading up a description object from a url. Normally
        you'll probably just want to pass a URL into the constructor.
        """
        req = Request(url, headers={'User-Agent':self.agent})
        self.dom = parse(urlopen(req))

        # version 1.1 has repeating Url elements
        self.urls = self._get_urls()

        # this is version 1.0 specific
        self.url = self._get_element_text('Url')
        self.format = self._get_element_text('Format')

        self.shortname = self._get_element_text('ShortName')
        self.longname = self._get_element_text('LongName')
        self.description = self._get_element_text('Description')
        self.image = self._get_element_text('Image')
        self.samplesearch = self._get_element_text('SampleSearch')
        self.developer = self._get_element_text('Developer')
        self.contact = self._get_element_text('Contact')
        self.attribution = self._get_element_text('Attribution')
        self.syndicationright = self._get_element_text('SyndicationRight')

        tag_text = self._get_element_text('Tags')
        if tag_text != None:
            self.tags = tag_text.split(" ")

        if self._get_element_text('AdultContent') == 'true':
            self.adultcontent = True
        else:
            self.adultcontent = False

    def get_url_by_type(self, type):
        """Walks available urls and returns them by type. Only 
        appropriate in opensearch v1.1 where there can be multiple
        query targets. Returns none if no such type is found.

        url = description.get_url_by_type('application/rss+xml')
        """
        for url in self.urls:
            if url.type == type:
                return url
        return None

    def get_best_template(self):
        """OK, best is a value judgement, but so be it. You'll get 
        back either the atom, rss or first template available. This
        method handles the main difference between opensearch v1.0 and v1.1
        """
        # version 1.0
        if self.url: 
            return self.url

        # atom
        if self.get_url_by_type('application/atom+xml'):
            return self.get_url_by_type('application/atom+xml').template

        # rss
        if self.get_url_by_type('application/rss+xml'):
            return self.get_url_by_type('application/rss+xml').template

        # other possible rss type
        if self.get_url_by_type('text/xml'):
            return self.get_url_by_Type('text/xml').template

        # otherwise just the first one
        if len(self.urls) > 0:
            return self.urls[0].template

        # out of luck
        return None
        

    # these are internal methods for querying xml

    def _get_element_text(self, tag):
        elements = self._get_elements(tag)
        if not elements:
            return None 
        return self._get_text(elements[0].childNodes)

    def _get_attribute_text(self, tag, attribute):
        elements = self._get_elements(tag)
        if not elements:
            return ''
        return elements[0].getAttribute('template')

    def _get_elements(self, tag):
        return self.dom.getElementsByTagName(tag)

    def _get_text(self, nodes):
        text = ''
        for node in nodes:
            if node.nodeType == node.TEXT_NODE:
                text += node.data
        return text.strip()

    def _get_urls(self):
        urls = []
        for element in self._get_elements('Url'):
            template = element.getAttribute('template')
            type = element.getAttribute('type')
            if template and type:
                url = URL()
                url.template = template
                url.type = type
                urls.append(url)
        return urls



class URL:
    """Class for representing a URL in an opensearch v1.1 query"""

    def __init__(self, type='', template='', method='GET'):
        self.type = type
        self.template = template
        self.method = 'GET'
        self.params = []



from description import Description
from query import Query
from results import Results

class Client:

    """This is the class you'll probably want to be using. You simply
    pass the constructor the url for the service description file and
    issue a search and get back results as an iterable Results object.

    The kinda neat thing about a Results object is that it will seamlessly
    handle fetching more results from the opensearch server when it can...
    so you just need to iterate and can let the paging be taken care of 
    for you.

    >>> from opensearch import Client
    >>> client = Client(description_url)
    >>> results = client.search("computer")
    >>> print results.totalResults
    >>> for result in results:
            print result.title

    If you have additional paramters to pass in the template just use
    their name. So for example if the template was 

        http://example.org/search?q={searchTerms}&key={key}

    you could supply key with:

    >>> results = client.search("computer", key="abc123")

    Additionally if you want to use prefixed parameter names you'll need to use
    the double underscore syntax. So for a template like:

        http://example.org/search?q={searchTerms}&key={ex:key}

    you could supply ex:key with:

    >>> results = client.search("computer", ex__key="abc123")
    """

    def __init__(self, url, agent="python-opensearch <https://github.com/edsu/opensearch>"):
        self.agent = agent
        self.description = Description(url, self.agent)

    def search(self, search_terms, page_size=25, **kwargs):
        """Perform a search and get back a results object
        """
        url = self.description.get_best_template()
        query = Query(url)

        # set up initial values
        query.searchTerms = search_terms
        query.count = page_size

        # add any additional parameters to the query
        for name, value in kwargs.items():
            setattr(query, name, value)

        # run the results
        return Results(query, agent=self.agent)




from opensearch import Client
from unittest import TestCase

class ClientTests(TestCase):

    url = 'http://www.koders.com/search/KodersSourceCodeSearchDescription.xml'

    def test_query(self):
        client = Client(ClientTests.url)
        self.assertEqual(client.agent, 'python-opensearch <https://github.com/edsu/opensearch>')
        results = client.search("computer")
        self.assertTrue(results.totalResults > 0)



import sys
from distutils.core import setup
from setuptools import find_packages

classifiers = """\
Intended Audience :: Education
Intended Audience :: Developers
Intended Audience :: Information Technology
License :: OSI Approved :: GNU General Public License (GPL)
Programming Language :: Python
Topic :: Text Processing :: General
"""

setup( 
    name             = 'opensearch',
    version          = '0.9.2',
    url              = 'http://github.com/edsu/opensearch',
    author           = 'Ed Summers',
    author_email     = 'ehs@pobox.com',
    license          = 'http://www.opensource.org/licenses/gpl-license.php',
    packages         = find_packages(exclude=['test']),
    description      = "Interact with opensearch services",
    classifiers      = filter( None, classifiers.split("\n") ),
    tests_require    = ["nose>=0.10.4"],
    test_suite       = 'nose.collector',
)

